@@ -13,6 +13,7 @@ Currently only Avro serialization is supported.
 """
 
 from .avro import AvroReader, AvroWriter
+import json
 import pandas as pd
 
 
@@ -25,7 +26,11 @@ def read_dataframe(client, hdfs_path):
   """
   with AvroReader(client, hdfs_path) as reader:
     # Hack-ish, but loading all elements in memory first to get length.
-    return pd.DataFrame.from_records(list(reader))
+    if 'pandas.columns' in reader.metadata:
+      columns = json.loads(reader.metadata['pandas.columns'])
+    else:
+      columns = None
+    return pd.DataFrame.from_records(list(reader), columns=columns)
 
 
 def write_dataframe(client, hdfs_path, df, **kwargs):
@@ -38,6 +43,7 @@ def write_dataframe(client, hdfs_path, df, **kwargs):
     :class:`hdfs.ext.avro.AvroWriter`.
 
   """
-  with AvroWriter(client, hdfs_path, **kwargs) as writer:
+  metadata = {'pandas.columns': json.dumps(df.columns.tolist())}
+  with AvroWriter(client, hdfs_path, metadata=metadata, **kwargs) as writer:
     for _, row in df.iterrows():
       writer.write(row.to_dict())
