@@ -13,7 +13,7 @@ import os.path as osp
 import sys
 
 try:
-  from hdfs.ext.avro import (_SeekableReader, _infer_schema, AvroReader,
+  from hdfs.ext.avro import (_SeekableReader, _SchemaInferrer, AvroReader,
     AvroWriter)
   from hdfs.ext.avro.__main__ import main
 except ImportError:
@@ -54,32 +54,59 @@ class TestSeekableReader(object):
 
 class TestInferSchema(object):
 
-  def _assert_schemas_equal(self, schema1, schema2):
-    # Only works for flat records.
-    ok_('fields' in schema1 and 'fields' in schema2)
-    fields = lambda schema: sorted(schema['fields'], key=lambda f: f['name'])
-    eq_(fields(schema1), fields(schema2))
+  def setup(self):
+    if SKIP:
+      raise SkipTest
 
-  def test_flat_record(self):
-    self._assert_schemas_equal(
-      _infer_schema({'foo': 1, 'bar': 'hello'}),
+  def test_array(self):
+    eq_(
+      _SchemaInferrer().infer({'foo': 1, 'bar': ['hello']}),
       {
         'type': 'record',
+        'name': '__Record1',
         'fields': [
+          {'type': {'type': 'array', 'items': 'string'}, 'name': 'bar'},
           {'type': 'int', 'name': 'foo'},
-          {'type': 'string', 'name': 'bar'},
         ]
       }
     )
 
-  def test_array(self):
-    self._assert_schemas_equal(
-      _infer_schema({'foo': 1, 'bar': ['hello']}),
+  def test_flat_record(self):
+    eq_(
+      _SchemaInferrer().infer({'foo': 1, 'bar': 'hello'}),
       {
         'type': 'record',
+        'name': '__Record1',
         'fields': [
+          {'type': 'string', 'name': 'bar'},
           {'type': 'int', 'name': 'foo'},
-          {'type': {'type': 'array', 'items': 'string'}, 'name': 'bar'},
+        ]
+      }
+    )
+
+  def test_nested_record(self):
+    eq_(
+      _SchemaInferrer().infer({'foo': {'bax': 2}, 'bar': {'baz': 3}}),
+      {
+        'type': 'record',
+        'name': '__Record1',
+        'fields': [
+          {
+            'type': {
+              'type': 'record',
+              'name': '__Record2',
+              'fields': [{'type': 'int', 'name': 'baz'}]
+            },
+            'name': 'bar',
+          },
+          {
+            'type': {
+              'type': 'record',
+              'name': '__Record3',
+              'fields': [{'type': 'int', 'name': 'bax'}]
+            },
+            'name': 'foo',
+          },
         ]
       }
     )
@@ -121,6 +148,27 @@ class TestRead(_AvroIntegrationTest):
     with AvroReader(self.client, 'weather.avro') as reader:
       eq_(list(reader), self.records)
 
+  def test_read_with_same_schema(self):
+    self.client.upload('w.avro', osp.join(self.dpath, 'weather.avro'))
+    with AvroReader(self.client, 'w.avro', reader_schema=self.schema) as reader:
+      eq_(list(reader), self.records)
+
+  def test_read_with_compatible_schema(self):
+    self.client.upload('w.avro', osp.join(self.dpath, 'weather.avro'))
+    schema = {
+      'name': 'test.Weather',
+      'type': 'record',
+      'fields': [
+        {'name': 'temp', 'type': 'int'},
+        {'name': 'tag', 'type': 'string', 'default': ''},
+      ],
+    }
+    with AvroReader(self.client, 'w.avro', reader_schema=schema) as reader:
+      eq_(
+        list(reader),
+        [{'temp': r['temp'], 'tag': ''} for r in self.records]
+      )
+
 
 class TestWriter(_AvroIntegrationTest):
 
@@ -145,7 +193,7 @@ class TestWriter(_AvroIntegrationTest):
       self.client,
       'weather.avro',
       schema=self.schema,
-      sync_interval = 1 # Flush block on every write.
+      sync_interval=1 # Flush block on every write.
     )
     with writer:
       for record in self.records:
